package editor;

import java.awt.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.ListIterator;
import java.util.Vector;

/**Reprsentiert einen Robotertyp mit einem bestimmten Antrieb, einer beliebigen
 *Anzahl von Sensoren, und einigen spezifischen Eigenschften
 * 
 */
public class RoboterTyp {
	private String name= new String();
    private long energievorrat;
    private int laenge; // auf Spielfeld
    private int breite;
    private long gewicht;
	private int sendereichweite;
    private Vector sensoren = new Vector();
    private Image bild;
    private AntriebsTyp antrieb;
    private static Vector robots=new Vector();


    /**Erzeugt ein neues RoboterTyp-Objekt
     *@param name der Name des Roboters
     *@param laenge Die L&auml;nge des Roboters
     *@param breite Die Breite des Roboters
     *@param gewicht Das Gewicht des Roboters exklusive Sensoren und 
     *Antriebsgewicht
     *@param energievorrat Der Anfangsenergievorrat des Roboters
     *@param sendereichweite Die maximale Sendereichweite des Roboters - 
     *wird bisher nicht verwendet
    */
     public RoboterTyp(String name, int laenge, int breite, long gewicht, long energievorrat, int sendereichweite, Image bild, AntriebsTyp antrieb){
        this.name=name;
        this.laenge=laenge;
        this.breite=breite;
        this.gewicht=gewicht;
        this.energievorrat = energievorrat;
        this.sendereichweite = sendereichweite;
        this.bild=bild;
        this.antrieb=antrieb;
        robots.add(this);

    }
	
     /**ldt einen Robotertyp anhand seines Namens aus der Datenbank
      *@param dbName der Name des Robotertypen
      *@return der geladene RoboterTyp
      */
	public static RoboterTyp loadRoboterTyp(String dbName) {
		//Folgender Absatz ist evtl. berflssig:
		Enumeration enum = robots.elements();
		while (enum.hasMoreElements()) {
			RoboterTyp rt = ((RoboterTyp) enum.nextElement());
            if (rt.getname().equals(dbName)) {
				return rt;
			} //Ende if
		} //Ende while
		System.out.println(dbName);
		RoboterTyp ret;
		Sql sql = new Sql();
		ResultSet rs = sql.select(
			"SELECT *\n" +
			"FROM roboter\n" +
			"WHERE name = '" + dbName + "';"
		);
		try {
			rs.first();
			String name = dbName;
			int rid = rs.getInt("rid");
			
			//Primitive ziehen;
			long energievorrat = rs.getLong("energievorrat");
			int sendereichweite = rs.getInt("sendereichweite");
			int laenge = rs.getInt("laenge");
			int breite = rs.getInt("breite");
			long gewicht = rs.getLong("gewicht");
			
			Image bild = Toolkit.getDefaultToolkit().getImage(rs.getString("bild"));
			
			//Antrieb ziehen
			int aid = rs.getInt("aid");
			AntriebsTyp antrieb = AntriebsTyp.loadAntriebsTyp(aid);
			
			//Sensoren Ziehen
			rs = sql.select(
				"SELECT sid\n" +
				"FROM rszuordnung\n" +
				"WHERE rid = " + rid +";"
			);
			rs.beforeFirst();
			ret = new RoboterTyp(name, laenge, breite, gewicht, energievorrat, sendereichweite, bild, antrieb);
            while(rs.next()) {
				int sid = rs.getInt("sid");
				ret.sensoren.add(SensorTyp.getSensorTyp(sid));
			}
			rs.close();
		    return ret;
		} catch (SQLException e) {
			e.printStackTrace();
		}
        return null;
	}


    /**Gibt das Gesamtgewicht aller am Roboter montierten Sensoren zurck
     *@return das Gesamtgewicht aller Sensoren
     */
    public int getSensorenGesamtGewicht() {
    	ListIterator li = sensoren.listIterator();
    	int sggw = 0;
    	while (li.hasNext()) {
    		sggw += ((SensorTyp) li.next()).getgewicht();
    	}
    	return sggw;
    }
    
    /**Gibt ein Array aller bekannten Robotertypen zur&uuml;ck
     *@return ein Array, welches alle bekannten RoboterTyp-Objekte enth&auml;lt
     */
    public static RoboterTyp[] getroboter(){
        return (RoboterTyp[]) robots.toArray(new RoboterTyp[robots.size()]);
    }

    /**Gibt den Namen des Roboters zur&uuml;ck
     *@return der Name des Roboters
     */
    public String getname(){
        return new String(name);
    }

    /**Setzt den Namen des Robotertypen
     *@param name der neue Name des Robotertypen
     */
    public void setname(String name){
        this.name = new String(name);
    }

    /**Gibt das Gewicht exklusaive Antriebsgewicht und Sensorengewicht 
     *zur&uuml;ck
     *@return das Gewicht dieses Robotertypenexklusaive Antriebsgewicht und 
     *Sensorengewicht
     */
    public long getgewicht(){
        return gewicht;
    }

    /**Setzt das Gewicht des Robotertypen
     *@param gewicht das neue Gewicht
     */
    public void setgewicht(int gewicht){
        this.gewicht = gewicht;
    }

    /**Setzt die Breite des Robotertypen
     *@param breite die neue Breite des Robotertypen
     */
    public void setbreite(int breite){
        this.breite=breite;
    }

    /**gibt die Breite des Robotertypen zur&uuml;ck
     *@return die Breite des Robotertypen
     */
    public int getbreite(){
        return breite;
    }

    /**Setzt die L&auml;nge des Robotertypen
     *@param laenge die neue L&auml;nge des Robotertypen
     */
    public void setlaenge(int laenge){
        this.laenge=laenge;
    }

    /**gibt die L&auml;nge des Robotertypen zur&uuml;ck
     *@return die L&auml;nge des Robotertypen
     */
    public int getlaenge(){
        return laenge;
    }

    /**Gibt eine Image-Repr&auml;sentation des Robotertypen zur&uuml;ck
     *@return ein Bild des Robotertypen
     */
    public Image getbild(){
        return bild;
    }

    /**Weist dem Robotertypen ein neues Bild zu
     *@param bild das neue Bild des Robotertypen
     */
    public void setbild(Image bild){
        this.bild=bild;
    }

    /**Gibt den Typ des eingebauten Antriebs zur&uuml;ck
     *@return der Typ des Antriebs
     */
    public AntriebsTyp getAntriebsTyp(){
        return antrieb;
    }

    /**Weist dem Robotertypen einen Antriebstypen zu
     *@param antrieb der neue Antriebstyp
     */
    public void setantrieb(AntriebsTyp antrieb){
        this.antrieb=antrieb;
    }

    /**Gibt ein Array mit den Typen aller an diesem Robotertyp zu befestigenden 
     *Sensoren zur&uuml;ck.
     *@return das Array mit den Sensortypen
     */
    public SensorTyp[] getSensorTypen(){
        return (SensorTyp[]) sensoren.toArray(new SensorTyp[sensoren.size()]);
    }

    /**F&uuml;gt dem Robotertypen einen neuen Sensortypen hinzu.
     *@param sensor der hinzuzuf&uuml;gende sensor
     */
    public void setsensor(SensorTyp sensor){
        sensoren.add(sensor);
    }

    /**Entfernt einen Sensortypen aus diesem Robotertypen
     *@param Sensortyp der zu entfernende Sensortyp
     */
    public void deletesensor(SensorTyp sensor){
        sensoren.remove(sensor);
    }

    /*&Auml;ndert den Anfangsenrgievorrat des Robotertypen
     *@param energievorrat der neue Anfangsenergievorrat
     */
    public void setenergievorrat(long energievorrat){
        this.energievorrat=energievorrat;
    }

    /**Gibt den Anfangsenergievorrat des Robotertypen zur&uuml;ck
     *@return der Anfangsenergievorrat
     */
    public long getenergievorrat(){
        return energievorrat;
    }

    /**Erzeugt eine Stringrepr&auuml;sentation des Robotertypen
     *@return eine Stringrepr&auml;sentation des Robotertypen
     */
	public String toString() {
		return
			"name: " + name + "\n" +
			"energievorrat: " + energievorrat + "\n";
	}


}
