/*
 * Created on 25.10.2003
 */
package gameserver;

import java.awt.Point;

import editor.SensorTyp;
import editor.Spielfeld;
import mailbox.ScanResult;
/**
 * @author Boris
 */
public class Sensor {
	private final int NULLWERT;
	private static final int SCHABLONENGROESSE = 21;
	private static final int OUTOFSPIELFELD = Integer.MAX_VALUE;
	private SensorTyp typ;
	private Roboter robby;
	private boolean scanning = false;

	public Sensor(SensorTyp typ, Roboter robby) {
		this.typ = typ;
		this.robby = robby;
		NULLWERT = (int) typ.getsichtbereiche() + 1;
	}

	private int[][] schablonenKopie(int[][] s) {
		int[][] ret = new int[s.length][s[0].length];
		for (int i = 1; i < s.length; i++)
			for (int j = 1; j < s[0].length; j++)
				ret[i][j] = s[i][j];
		return ret;
	}

	public double dist(int x1, int y1, int x2, int y2) {
		return Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
	}

	/*Dies ist ein Schnes Relikt:
	private int[] nextField(int aktX, int aktY, int zielX, int zielY) {
		double bestDist = 0;
		int bestI = 0;
		int bestJ = 0;
		for (int i = -1; i <= 1; i++) {
			for (int j = -1; j <= 1; j++) {
				double d =
					(dist(aktX, aktY, zielX, zielY)
						- dist(aktX + i, aktY + j, zielX, zielY))
						/ Math.sqrt(i * i + j * j);
				if ((i == 0) && (j == 0))
					d = 0;
				//System.out.println(d);
				if (d > bestDist) {
					bestDist = d;
					bestI = i;
					bestJ = j;
				}
			}
		}
		int[] punkt = { aktX + bestI, aktY + bestJ };
		return punkt;
	}
	*/
	public Point[] getDirekterPfad(int startX, //Startposition
	int startY, int zielX, //Zielposition
	int zielY) {
		//Maximum aus X-Differenz und Y-Differenz speichern:
		int maxLaenge =
			Math.max(Math.abs(zielX - startX), Math.abs(zielY - startY));
		Point[] points = new Point[maxLaenge + 1]; // Punktearray

		//Steigungen berechnen (eine ist 1)
		double xSteigung = (double) (zielX - startX) / maxLaenge;
		double ySteigung = (double) (zielY - startY) / maxLaenge;
		//Pfad berechnen:
		for (int i = 0; i <= maxLaenge; i++) {
			double xGenau = startX + i * xSteigung;
			double yGenau = startY + i * ySteigung;
			double xGerundet = Math.round(xGenau);
			double yGerundet = Math.round(yGenau);
			xGerundet =
				(xGerundet - xGenau == 0.5) ? points[i - 1].getX() : xGerundet;
			yGerundet =
				(yGerundet - yGenau == 0.5) ? points[i - 1].getY() : yGerundet;
			points[i] = new Point((int) xGerundet, (int) yGerundet);
		}

		return points;
	}

	/*Holt sich erst einen direkten Pfad zwischen zwei Punkten und
	 * trgt darauf die Hhen ab. Bercksichtigt Durchsichtigkeit.
	 */

	private int[] getHoehenPfad(int xStart, int yStart, int xZiel, int yZiel) {
		Spielfeld spielfeld = robby.getSpielfeld();
		Point[] points = getDirekterPfad(xStart, yStart, xZiel, yZiel);
		int[] ret = new int[points.length];
		for (int arrayIndex = 0; arrayIndex < points.length; arrayIndex++) {
			//Koordinaten holen:
			int aktX = points[arrayIndex].x;
			int aktY = points[arrayIndex].y;
			try {
				//Entscheiden ob durchsichttiges Objekt oder nicht:
				if (typ
					.getdurchsichtigkeit(spielfeld.getgelaende(aktX, aktY))) {
					ret[arrayIndex] = spielfeld.gethoehe(aktX, aktY);
				} else {
					ret[arrayIndex] =
						spielfeld.gethoehe(aktX, aktY)
							+ spielfeld.getObjektHoehe(aktX, aktY);
				} //Ende if...else

			} catch (RuntimeException e) {
				e.printStackTrace();
			}
		}
		return ret;
	}

	/**Fhrt einen Scan aus.<br>
         *Bercksichtigt die Richtung des zugehrigen Roboterobjektes, welches 
         *im Konstruktor angegeben wurde.
         *ber die sendScanResult-Methode des Roboterobjektes wird das Resultat 
         *des Scans verschickt.
         */
        public synchronized void doScan() {
		scanning = true;
		new Thread() {
			public void run() {
				long scanTime = 0;
				long startTime = System.currentTimeMillis();
				int l = typ.getschablone().length;
				int[][] schablone = new int[l][l];
				for (int i = 0; i < l; i++) {
					for (int j = 0; j < l; j++) {
						schablone[i][j] =
							typ.getschablone()[i][j] == 0
								? NULLWERT
								: typ.getschablone()[i][j];
					}
				}

				schablone = getGedrehteSchablone(getWinkel(), schablone);

				String[][] ret =
					new String[schablone.length][schablone[0].length];
				Spielfeld spielfeld = robby.getSpielfeld();

				//Testweise schablone dumpen:
				/*
				//log("Schablone:");
				for (int i = 0; i < l; i++) {
					for (int j = 0; j < l; j++) {
						System.out.print(schablone[j][i]);
					}
					System.out.println();
				}*/
				/*
				//Testweise Ausgabe der Hhen
				log("Hhen:");
				for (int i = 0; i < 20; i++) {
					for (int j = 0; j < 20; j++) {
						System.out.print(spielfeld.gethoehe(j, i));
					}
					System.out.println();
				}*/

				/*
				//Testweise Ausgabe der ObjektHhen
				log("Objekthen:");
				for (int i = 0; i < 20; i++) {
					for (int j = 0; j < 20; j++) {
						System.out.print(spielfeld.getObjektHoehe(j, i));
					}
					System.out.println();
				}*/

				//Position auf dem Spielfeld holen:
				int x = robby.getX();
				int y = robby.getY();

				/*********************************************
				* verdeckte Felder aus der Schablone lschen:*
				*Algorithmus: Wenn die Mitte sichtbar ist,   *
				*ist das Feld sichtbar. Also Gerade von der  *
				*Mitte zum Sensor ziehen. Falls Hindernis    *
				*schneidet -> unsichtbar                     *
				*********************************************/
				//Hhenberechnung: Berechnet relevante Hhe(nach Durchsichtigkeit) jedes Feldes im Sichtbereich

				for (int i = 0; i < l; i++) {
					for (int j = 0; j < l; j++) {
						//Schablone auf Spielfeld projezieren:
						int xPos = x + i - l / 2;
						int yPos = y + j - l / 2;

						//Im Spielfeld? dann los
						if ((xPos >= 0)
							&& (yPos >= 0)
							&& (xPos < spielfeld.getBreite())
							&& (yPos < spielfeld.getLaenge())) {
							//Wenn laut Schablone sichtbar
							if ((schablone[i][j] != NULLWERT)
								&& ((xPos != x) || (yPos != y))) {
								//Array mit Hhen auf dem Weg zum Punkt holen
								int[] hoehen = getHoehenPfad(x, y, xPos, yPos);
								//Jetzt maximale Steigung *vor* dem Punkt berechnen:
								double steigung = Double.NEGATIVE_INFINITY;
								for (int k = 1; k < hoehen.length - 1; k++) {
									double aktSteigung =
										(double) (hoehen[k] - spielfeld.gethoehe(x, y)) / k;
									if (aktSteigung > steigung)
										steigung = aktSteigung;
								}

							
								if ((hoehen[0] + hoehen.length * steigung
									> spielfeld.gethoehe(xPos, yPos)
                                                                        + spielfeld.getObjektHoehe(xPos, yPos)) || ((spielfeld.getObjektHoehe(x, y)>0 ) && (!typ.getdurchsichtigkeit(spielfeld.getgelaende(x, y))))) {
									schablone[i][j] = NULLWERT;
								};

							}

						} else {
							schablone[i][j] = OUTOFSPIELFELD;

						}
					}

				}

				//Sensormeldungen in return-array kopieren
				for (int i = 0; i < schablone.length; i++) {
					for (int j = 0; j < schablone[0].length; j++) {
						int sij = schablone[i][j];
						if (sij != OUTOFSPIELFELD) {
							if (sij != NULLWERT) {
								int absX = x + i - l / 2;
								int absY = y + j - l / 2;
								ret[i][j] =
									spielfeld.getSensormeldung(
										typ.getName(),
										sij,
										absX,
										absY);
                                    spielfeld.setfog(absX, absY, false);
								scanTime
									+= typ.getscandauer(
										spielfeld.getgelaende(absX, absY));

							} else {
								//log (""+ sij + " wurde nicht gesehen");
							}
						} else {
							ret[i][j] = ScanResult.OUTOFSPIELFELD;
						} //Ende else

					} //Ende for
				} //Ende for

				long timeUntilNow = System.currentTimeMillis() - startTime;
				long stillToWait = scanTime - timeUntilNow;
				if(stillToWait < 0) stillToWait = 0;
				System.out.println(timeUntilNow);
				System.out.println(stillToWait);
				try {
					Thread.sleep(stillToWait);
				} catch (InterruptedException e) {
				}
				robby.decreaseEnergievorrat(typ.getEnergieVerbrauchProScan());
				scanning = false;
				robby.sendScanResult(typ.getName(), ret);
                spielfeld.repaint();
			} //Ende run
		} //Ende Thread
		.start();
	} //Ende doScan

	private int[][] getGedrehteSchablone(double winkel, int[][] schablone) {
		int size = SCHABLONENGROESSE;
		float middle = (size / 2);
		float newMiddle = (size / 2);
		//Eigentlich rotiert man die neue Matrix zurck, um die Punkte zu bestimmen.
		int[][] ret = new int[size][size];

		for (int rawX = 0; rawX < size; rawX++) {
			for (int rawY = 0; rawY < size; rawY++) {
				double x = rawX - newMiddle;
				double y = rawY - newMiddle;
				//Quellkoordinaten berechnen
				//Die Drehformeln erhlt man durch die Additionstheoreme:
				double sourceX =
					x * Math.cos(winkel) - y * Math.sin(winkel) + middle;
				double sourceY =
					y * Math.cos(winkel) + x * Math.sin(winkel) + middle;
				if ((sourceX >= 0)
					&& (sourceX <= size - 1)
					&& (sourceY >= 0)
					&& (sourceY <= size - 1)) {
					//Dann kopieren:
					ret[rawX][rawY] =
						getInterpolierterWert(sourceX, sourceY, schablone);
				} else {
					ret[rawX][rawY] = NULLWERT; // sonst nix eintragen
				}
			}
		}
		return ret; //fertich!!!
	}
	private int getInterpolierterWert(double x, double y, int[][] schablone) {
		int[] pointsX = new int[2];
		double[] weightsX = new double[2];
		int[] pointsY = new int[2];
		double[] weightsY = new double[2];
		pointsX[0] = (int) Math.floor(x);
		pointsX[1] = (int) Math.ceil(x);
		//if (pointsX[0] <  0) pointsX[0] = 0;
		//if (pointsX[1] > data.length) pointsX[1] = data.length;
		weightsX[0] = pointsX[1] - x;
		weightsX[1] = 1 - weightsX[0];
		pointsY[0] = (int) Math.floor(y);
		pointsY[1] = (int) Math.ceil(y);
		weightsY[0] = pointsY[1] - y;
		weightsY[1] = 1 - weightsY[0];
		double ret = 0;
		for (int i = 0; i < 2; i++)
			for (int j = 0; j < 2; j++)
				try {
					ret += schablone[pointsX[i]][pointsY[j]]
						* weightsX[i]
						* weightsY[j];
				} catch (Exception e) {
					System.out.println(
						pointsX[0]
							+ ", "
							+ pointsX[1]
							+ ", "
							+ pointsY[0]
							+ ", "
							+ pointsY[1]);
					System.out.println(x + ", " + y);
				}
		return (int) Math.round(ret);
	}

	private double getWinkel() {
		String richtung = robby.getBlickRichtung();
		final double PI = Math.PI;
		if (richtung.equals("Nord"))
			return 0;
		if (richtung.equals("Nordwest"))
			return PI / 4;
		if (richtung.equals("West"))
			return PI / 2;
		if (richtung.equals("Sdwest"))
			return PI * 3 / 4;
		if (richtung.equals("Sd"))
			return PI;
		if (richtung.equals("Sdost"))
			return PI * 5 / 4;
		if (richtung.equals("Ost"))
			return PI * 3 / 2;
		if (richtung.equals("Nordost"))
			return PI * 7 / 4;
		return (Double.NaN);
	}

	public String getTypName() {
		return typ.getName();
	}

	/**
	 * @param replayID

	public void writeToReplayDB(int replayID) {
		typ.writeToReplayDB(replayID);

	}
     */

	/*
	public void log(String message) {
		System.out.println("Sensor.java :  " + message);
	}*/
	boolean isScanning(){
		return scanning;
	}
}
