package mailbox;




import agent.Agent;

import java.util.Vector;

import mailbox.AgentMailbox;
import editor.MessageFrame;

/**
 *
 * User: Heiner Frhling
 * Date: 26.10.2003
 * Time: 17:20:40
 *
 */
public class MailboxSystem implements ServerMailbox{

    private Vector ivServerMailbox;
    private Vector ivAgentenMailboxen;
    private MessageFrame ivMessageFrame;
    private boolean ivSystemRunning = false;



    public MailboxSystem(){
        ivServerMailbox = new Vector();
        ivAgentenMailboxen = new Vector();
    }



    /**
     * Nimmt ein Array von Agenten entgegen, erzeugt neue Mailboxen und
     * fgt sie in die eigene Verwaltung ein. Anschlieend wird die Mailbox
     * beim Agenten registriert.
     * @param agents
     */
    public void initMailboxSystem(Agent[] agents){

        for (int i = 0; i < agents.length; i++){
            AgentMailbox m = new AgentMailbox(agents[i], this);
            ivAgentenMailboxen.add(m);
            agents[i].setAgentMailbox(m);
        }
    }


    /**
     * Ermglicht dem Server, eine Nachricht an alle Agenten zu verschicken
     *
     * @param msg Die an alle  Agenten zu verteilende Nachricht
     */
    public void sendBroadcast(Message msg) {
        if (ivSystemRunning) {
            msg.setAbsender(Message.SERVER);
            msg.setEmpfaenger("ALL AGENTS");
            for (int i = 0; i < ivAgentenMailboxen.size(); i++){
               ((AgentMailbox) ivAgentenMailboxen.get(i)).appendMessage(msg);
            }

            ivMessageFrame.log(msg);
        }
    }

    /**
     * Versendet eine Nachricht an alle AgentenMailboxen auer an die des
     * Absenders
     *
     * @param msg Die an alle anderen Agenten zu verteilende Nachricht
     */
    public synchronized void sendBroadcastBetweenAgents(Message msg){
        msg.setEmpfaenger("BROADCAST BETWEEN AGENTS");
        if (ivSystemRunning) {
            for (int i = 0; i < ivAgentenMailboxen.size(); i++){
                if (!((AgentMailbox) ivAgentenMailboxen.get(i)).getOwnerName().equals(msg.getAbsender())){
                    ((AgentMailbox) ivAgentenMailboxen.get(i)).appendMessage(msg);
                }
            }
            ivMessageFrame.log(msg);
        }
    }

    /**
     * Kopiert die bergebene Nachricht in die Message-Queue des Servers
     * @param msg Die zu versendende Nachricht
     */
    public synchronized void sendMessageToServer(Message msg) {

        if (ivSystemRunning) {
            msg.setEmpfaenger("GAMESERVER");
            ivServerMailbox.add(msg);
            ivMessageFrame.log(msg);
        }
    }


    /**
     * Kopiert die bergebene Nachricht in die Message-Queue als Empfnger
     * angegebenen Agenten
     * @param msg Die zu versendende Nachricht
     */
    public synchronized void sendAgentMessageToAgent(Message msg){
        if (ivSystemRunning) {
            boolean errormessage = true;

            for (int i = 0; i < ivAgentenMailboxen.size(); i++){
                if (((AgentMailbox) ivAgentenMailboxen.get(i)).getOwnerName().equals(msg.getEmpfaenger())){
                    errormessage = false;
                    ((AgentMailbox) ivAgentenMailboxen.get(i)).appendMessage(msg);
                    ivMessageFrame.log(msg);
                }
            }

            if (errormessage){
                sendAgentMessageToAgent(new Message(Message.MAILBOXSYSTEM ,  msg.getAbsender(), Message.ERR_AGENT_UNKOWN, null));
            }
        }

    }


    /**
     * NUR VOM SERVER ZU BENUTZENDE METHODE
     *
     * Abrufmethode fr Servernachrichten (FIFO-Prinzip)
     * @return Die nchste Nachricht in der Mailbox oder null, falls keine vorliegt
     */
    public synchronized Message getNextServerMessage() {
        Message lvMessage = null;

        if (ivServerMailbox.size() == 0){
            // tue explizit nichts, lvMessage bleibt auf null
        }
        else {
            lvMessage =  (Message)  ivServerMailbox.remove(0);
        }

        return lvMessage;
    }


    /**
     * NUR VOM SERVER ZU BENUTZENDE METHODE
     *
     * Kopiert die bergebene Nachricht in die Message-Queue als Empfnger
     * angegebenen Agenten
     * @param msg Die zu versendende Nachricht
     * @param agentID Der Empfnger
     */
    public synchronized void sendMessageToAgent(String agentID, Message msg) {
        if (ivSystemRunning) {
            msg.setAbsender(Message.SERVER);
            msg.setEmpfaenger(agentID);
            for (int i = 0; i < ivAgentenMailboxen.size(); i++){

                if (((AgentMailbox) ivAgentenMailboxen.get(i)).getOwnerName().equals(agentID)){
                    ((AgentMailbox) ivAgentenMailboxen.get(i)).appendMessage(msg);
                    ivMessageFrame.log(msg);
                }
            }
        }

    }


    /**
     * Entfernt die Mailbox des Agenten und wird benutzt, falls der SimulationCotroller
     * einen Agenten entfernen muss, der auf eine Mine gefahren ist.
     * @param agentName
     */
    public synchronized void removeAgentMailbox(String agentName){
        for (int i = 0; i < ivAgentenMailboxen.size(); i++){
             if (((AgentMailbox) ivAgentenMailboxen.get(i)).getOwnerName().equals(agentName)){
                 AgentMailbox a = (AgentMailbox) ivAgentenMailboxen.remove(i);
                 a = null;
             }
        }
    }

    /**
     * Setzt den MessageFrame, der in der SimulationGUI zum Loggen der ausgetauschten
     * Nachrichten verwendet wird.
     * @param ivMessageFrame
     */
    public void setMessageFrame(MessageFrame ivMessageFrame) {
        this.ivMessageFrame = ivMessageFrame;
    }

    /**
     * Derzeit nicht benutzt
     *
     */
    public boolean isSystemRunning() {
        return ivSystemRunning;
    }

    public void setSystemRunning(boolean ivSystemRunning) {
        this.ivSystemRunning = ivSystemRunning;
    }

}
